@@ -65,6 +65,7 @@ cVNSIClient::cVNSIClient(int fd, unsigned int id, const char *ClientAdr, CVNSITi
 {
   SetDescription("VNSI Client %u->%s", id, ClientAdr);
 
+  m_StatusInterfaceEnabled = false;
   Start();
 }
 
@@ -165,7 +166,7 @@ void cVNSIClient::Action(void)
 bool cVNSIClient::StartChannelStreaming(cResponsePacket &resp, const cChannel *channel, int32_t priority, uint8_t timeshift, uint32_t timeout)
 {
   delete m_Streamer;
-  m_Streamer    = new cLiveStreamer(m_Id, m_bSupportRDS, timeshift, timeout);
+  m_Streamer = new cLiveStreamer(m_Id, m_bSupportRDS, m_protocolVersion, timeshift, timeout);
   m_isStreaming = m_Streamer->StreamChannel(channel, priority, &m_socket, &resp);
   return m_isStreaming;
 }
@@ -226,6 +227,7 @@ int cVNSIClient::EpgChange()
     return callAgain;
 
 #if VDRVERSNUM >= 20301
+  LOCK_CHANNELS_READ;
   cStateKey SchedulesStateKey(true);
   const cSchedules *schedules = cSchedules::GetSchedulesRead(SchedulesStateKey);
   if (!schedules)
@@ -246,7 +248,6 @@ int cVNSIClient::EpgChange()
       continue;
 
 #if VDRVERSNUM >= 20301
-    LOCK_CHANNELS_READ;
     const cChannel *channel = Channels->GetByChannelID(schedule->ChannelID());
 #else
     Channels.Lock(false);
@@ -299,8 +300,6 @@ int cVNSIClient::EpgChange()
 
 void cVNSIClient::Recording(const cDevice *Device, const char *Name, const char *FileName, bool On)
 {
-  cMutexLock lock(&m_msgLock);
-
   if (m_StatusInterfaceEnabled)
   {
     cResponsePacket resp;
@@ -324,8 +323,6 @@ void cVNSIClient::Recording(const cDevice *Device, const char *Name, const char
 
 void cVNSIClient::OsdStatusMessage(const char *Message)
 {
-  cMutexLock lock(&m_msgLock);
-
   if (m_StatusInterfaceEnabled && Message)
   {
     /* Ignore this messages */
@@ -350,6 +347,7 @@ void cVNSIClient::OsdStatusMessage(const char *Message)
     else if (strcasecmp(Message, trVDR("Cancel editing?")) == 0) return;
     else if (strcasecmp(Message, trVDR("Cutter already running - Add to cutting queue?")) == 0) return;
     else if (strcasecmp(Message, trVDR("No index-file found. Creating may take minutes. Create one?")) == 0) return;
+    else if (strcasecmp(Message, trVDR("Low disk space!")) == 0) return;
     else if (strncmp(Message, trVDR("VDR will shut down in"), 21) == 0) return;
 
     cResponsePacket resp;
@@ -717,6 +715,8 @@ bool cVNSIClient::process_GetSetup(cRequestPacket &req) /* OPCODE 8 */
     resp.add_U32(TimeshiftBufferSize);
   else if (!strcasecmp(name, CONFNAME_TIMESHIFTBUFFERFILESIZE))
     resp.add_U32(TimeshiftBufferFileSize);
+  else if (!strcasecmp(name, CONFNAME_EDL))
+    resp.add_U32(EdlMode);
 
   resp.finalise();
   m_socket.write(resp.getPtr(), resp.getLen());
@@ -752,6 +752,11 @@ bool cVNSIClient::process_StoreSetup(cRequestPacket &req) /* OPCODE 9 */
     int value = req.extract_U32();
     cPluginVNSIServer::StoreSetup(CONFNAME_PLAYRECORDING, value);
   }
+  else if (!strcasecmp(name, CONFNAME_EDL))
+  {
+    int value = req.extract_U32();
+    cPluginVNSIServer::StoreSetup(CONFNAME_EDL, value);
+  }
 
   cResponsePacket resp;
   resp.init(req.getRequestID());
@@ -1006,17 +1011,25 @@ bool cVNSIClient::processRecStream_GetIFrame(cRequestPacket &req) /* OPCODE 45 *
 bool cVNSIClient::processRecStream_GetLength(cRequestPacket &req) /* OPCODE 46 */
 {
   uint64_t length = 0;
+  uint64_t lengthMsec = 0;
 
   if (m_RecPlayer)
   {
     m_RecPlayer->reScan();
     length = m_RecPlayer->getLengthBytes();
+
+    lengthMsec = m_RecPlayer->getLengthFrames() * 1000 / m_RecPlayer->getFPS();
   }
 
   cResponsePacket resp;
   resp.init(req.getRequestID());
   resp.add_U64(length);
 
+  if (m_protocolVersion >= 12)
+  {
+    resp.add_U64(lengthMsec);
+  }
+
   resp.finalise();
   m_socket.write(resp.getPtr(), resp.getLen());
 
@@ -1081,6 +1094,9 @@ bool cVNSIClient::processCHANNELS_GetChannels(cRequestPacket &req) /* OPCODE 63
     if (channel->Sid() == 0)
       continue;
 
+    if (endswith(channel->Name(), "OBSOLETE"))
+      continue;
+
     // check filter
     if (filter && !VNSIChannelFilter.PassFilter(*channel))
       continue;
@@ -1213,6 +1229,9 @@ bool cVNSIClient::processCHANNELS_GetGroupMembers(cRequestPacket &req)
     if(name.empty())
       continue;
 
+    if (endswith(channel->Name(), "OBSOLETE"))
+      continue;
+
     if(cVNSIChannelFilter::IsRadio(channel) != radio)
       continue;
 
@@ -2039,6 +2058,7 @@ bool cVNSIClient::processRECORDINGS_GetList(cRequestPacket &req) /* OPCODE 102 *
 {
   cMutexLock lock(&m_timerLock);
 #if VDRVERSNUM >= 20301
+  LOCK_CHANNELS_READ;
   LOCK_RECORDINGS_READ;
 #else
   cThreadLock RecordingsLock(&Recordings);
@@ -2053,11 +2073,7 @@ bool cVNSIClient::processRECORDINGS_GetList(cRequestPacket &req) /* OPCODE 102 *
   for (cRecording *recording = Recordings.First(); recording; recording = Recordings.Next(recording))
 #endif
   {
-#if APIVERSNUM >= 10705
     const cEvent *event = recording->Info()->GetEvent();
-#else
-    const cEvent *event = NULL;
-#endif
 
     time_t recordingStart    = 0;
     int    recordingDuration = 0;
@@ -2076,11 +2092,7 @@ bool cVNSIClient::processRECORDINGS_GetList(cRequestPacket &req) /* OPCODE 102 *
       }
       else
       {
-#if APIVERSNUM >= 10727
         recordingStart = recording->Start();
-#else
-        recordingStart = recording->start;
-#endif
       }
     }
     DEBUGLOG("GRI: RC: recordingStart=%lu recordingDuration=%i", recordingStart, recordingDuration);
@@ -2092,18 +2104,10 @@ bool cVNSIClient::processRECORDINGS_GetList(cRequestPacket &req) /* OPCODE 102 *
     resp.add_U32(recordingDuration);
 
     // priority
-#if APIVERSNUM >= 10727
     resp.add_U32(recording->Priority());
-#else
-    resp.add_U32(recording->priority);
-#endif
 
     // lifetime
-#if APIVERSNUM >= 10727
     resp.add_U32(recording->Lifetime());
-#else
-    resp.add_U32(recording->lifetime);
-#endif
 
     // channel_name
     resp.add_String(recording->Info()->ChannelName() ? m_toUTF8.Convert(recording->Info()->ChannelName()) : "");
@@ -2111,7 +2115,6 @@ bool cVNSIClient::processRECORDINGS_GetList(cRequestPacket &req) /* OPCODE 102 *
     {
       // channel uuid
 #if VDRVERSNUM >= 20301
-      LOCK_CHANNELS_READ;
       const cChannel *channel = Channels->GetByChannelID(recording->Info()->ChannelID());
 #else
       Channels.Lock(false);
@@ -2179,29 +2182,33 @@ bool cVNSIClient::processRECORDINGS_GetList(cRequestPacket &req) /* OPCODE 102 *
     std::string strDirectory;
     if (directory)
       strDirectory = directory;
-    int noOfEntries = 1;
-    char* filename = strdup(recording->FileName());
-    char *pch = strrchr(filename, '/');
-    if (pch)
+
+    if (GroupRecordings)
     {
-      int noOfRecs = 0;
-      *pch = 0;
-      char* foldername = filename;
-      struct dirent **fileListTemp;
-      noOfEntries = scandir(foldername, &fileListTemp, NULL, alphasort);
-      for (int i=0; i<noOfEntries; i++)
+      int noOfEntries = 1;
+      char* filename = strdup(recording->FileName());
+      char *pch = strrchr(filename, '/');
+      if (pch)
       {
-        std::string name(fileListTemp[i]->d_name);
-        if (name.find(".rec") != std::string::npos)
-          noOfRecs++;
-      }
-      if (noOfRecs > 1)
-      {
-        strDirectory += "/";
-        strDirectory += recname;
+        int noOfRecs = 0;
+        *pch = 0;
+        char* foldername = filename;
+        struct dirent **fileListTemp;
+        noOfEntries = scandir(foldername, &fileListTemp, NULL, alphasort);
+        for (int i=0; i<noOfEntries; i++)
+        {
+          std::string name(fileListTemp[i]->d_name);
+          if (name.find(".rec") != std::string::npos)
+            noOfRecs++;
+        }
+        if (noOfRecs > 1)
+        {
+          strDirectory += "/";
+          strDirectory += recname;
+        }
       }
+      free(filename);
     }
-    free(filename);
 
     resp.add_String(strDirectory.empty() ? "" : m_toUTF8.Convert(strDirectory.c_str()));
 
@@ -2335,13 +2342,43 @@ bool cVNSIClient::processRECORDINGS_GetEdl(cRequestPacket &req) /* OPCODE 105 */
     if(marks.Load(recording->FileName(), recording->FramesPerSecond(), recording->IsPesRecording()))
     {
 #if VDRVERSNUM >= 10732
-      cMark* mark = NULL;
       double fps = recording->FramesPerSecond();
-      while((mark = marks.GetNextBegin(mark)) != NULL)
+      cMark* mark = NULL;
+
+      if (EdlMode == 0)         /* scene */
       {
-        resp.add_U64(mark->Position() *1000 / fps);
-        resp.add_U64(mark->Position() *1000 / fps);
-        resp.add_S32(2);
+        while((mark = marks.GetNextBegin(mark)) != NULL)
+        {
+          resp.add_U64(mark->Position() *1000 / fps);
+          resp.add_U64(mark->Position() *1000 / fps);
+          resp.add_S32(2);
+        }
+      }
+      else
+      {
+        int kodiMode = EdlMode==1 ? 3 : 0; /* comskip : cut */
+        int endPos = 0;
+        cMark* endMark = NULL;
+
+        while ((mark = marks.GetNextBegin(endMark)) != NULL)
+        {
+          if (endPos != mark->Position())
+          {
+            resp.add_U64(endPos *1000 / fps);
+            resp.add_U64(mark->Position() *1000 / fps);
+            resp.add_S32(kodiMode);
+          }
+          if ((endMark = marks.GetNextEnd(mark)) == NULL)
+            break;
+          endPos = endMark->Position();
+        }
+
+        if (endMark && endPos < recording->NumFrames())
+        {
+          resp.add_U64(endPos *1000 / fps);
+          resp.add_U64(recording->NumFrames() *1000 / fps);
+          resp.add_S32(kodiMode);
+        }
       }
 #endif
     }
@@ -2835,11 +2872,7 @@ bool cVNSIClient::processRECORDINGS_DELETED_GetList(cRequestPacket &req) /* OPCO
       }
       else
       {
-#if APIVERSNUM >= 10727
         recordingStart = recording->Start();
-#else
-        recordingStart = recording->start;
-#endif
       }
     }
     DEBUGLOG("GRI: RC: recordingStart=%lu recordingDuration=%i", recordingStart, recordingDuration);
@@ -2851,18 +2884,10 @@ bool cVNSIClient::processRECORDINGS_DELETED_GetList(cRequestPacket &req) /* OPCO
     resp.add_U32(recordingDuration);
 
     // priority
-#if APIVERSNUM >= 10727
     resp.add_U32(recording->Priority());
-#else
-    resp.add_U32(recording->priority);
-#endif
 
     // lifetime
-#if APIVERSNUM >= 10727
     resp.add_U32(recording->Lifetime());
-#else
-    resp.add_U32(recording->lifetime);
-#endif
 
     // channel_name
     resp.add_String(recording->Info()->ChannelName() ? m_toUTF8.Convert(recording->Info()->ChannelName()) : "");
